@@ -12,7 +12,7 @@ from django.db import models, connection
 from django_orm.postgresql import hstore
 from django.utils.datastructures import SortedDict
 from utils import single_list_to_tuple
-
+import caching.base
 import copy
 from django.utils.text import capfirst
 from django import forms
@@ -21,6 +21,8 @@ import json
 FIELD_TYPES = ['Input', 'Monetary', 'Float', 'Integer', 'TextArea',
     'SelectBox', 'MultSelect', 'Date', 'DateTime', 'CheckBox', 'RadioButton']
 
+FIELD_TYPES_WITHOUT_BLANK_OPTION = ['MultSelect', 'CheckBox', 'RadioButton']
+
 FIELD_TYPES_DICT = dict(Input='models.CharField',
     Monetary='models.DecimalField',
     Float='models.FloatField',
@@ -32,26 +34,69 @@ FIELD_TYPES_DICT = dict(Input='models.CharField',
     DateTime='models.CharField',
     CheckBox='MultiSelectField',
     RadioButton='UncleanedCharField')
-    
+
 FIELD_TYPE_DEFAULT = 'models.CharField'
 
 
+class DynamicField(caching.base.CachingMixin, models.Model):
+    refer = models.CharField(max_length=120, blank=False, db_index=True, verbose_name="Class name")
+    name = models.CharField(max_length=120, blank=False, db_index=True, verbose_name="Field name")
+    verbose_name = models.CharField(max_length=120, blank=False, verbose_name="Verbose name")
+    typo = models.CharField(max_length=20, blank=False, db_index=True, verbose_name="Field type",
+        choices=single_list_to_tuple(FIELD_TYPES))
+    max_length = models.IntegerField(null=True, blank=True, verbose_name="Length")
+    blank = models.BooleanField(default=True, verbose_name="Blank")
+    choices = models.TextField(null=True, blank=True, verbose_name="Choices")
+    default_value = models.CharField(max_length=80, null=True, blank=True, verbose_name="Default value")
+
+    cache = caching.base.CachingManager()
+
+    class Meta:
+        db_table = u'dynamic_field'
+
+    @property
+    def has_blank_option(self):
+        return self.blank and \
+            self.typo not in FIELD_TYPES_WITHOUT_BLANK_OPTION
+
+
+
+
+# XXX: Charge memory with all dfields for prevent flood on db.
+# NOTE: this solution need to restart project on each new dfield add. Nasty!!
+dfields =  DynamicField.cache.all()
+def find_dfields(refer, name=None):
+    if name:
+        return [dfield for dfield in dfields \
+            if dfield.refer == refer and dfield.name == name]
+    return [dfield for dfield in dfields if dfield.refer == refer]
+
+# NOTE: Error happen on syncdb, because DynamicField's table does not exist.
+cursor = connection.cursor()
+cursor.execute("select count(*) from pg_tables where tablename='dynamic_field'")
+DYNAMIC_FIELD_TABLE_EXIST = (cursor.fetchone()[0] > 0)
+
+
 class UncleanedCharField(models.CharField):
     def clean(self, value, *args):
         # ignore clean
         return value
-        
+
     def get_choices(self, include_blank=False):
         """
-        Overriding the method to remove the 
+        Overriding the method to remove the
         BLANK_OPTION in Checkbox, Radio and Select.
-        
+
         * Only if the dfield is blank
         """
         choices = []
-        if self.blank:
+
+        # FIXME: this maybe mistake with fields with same name in different refers
+        dynamic_field = DynamicField.cache.get(name=self.name)
+
+        if dynamic_field.has_blank_option:
             choices = super(UncleanedCharField, self).get_choices()
-        
+
         return choices or self._choices
 
 
@@ -92,36 +137,6 @@ class MultiSelectField(UncleanedCharField):
         defaults.update(kwargs)
         return form_class(**defaults)
 
-class DynamicField(models.Model):
-    refer = models.CharField(max_length=120, blank=False, db_index=True, verbose_name="Class name")
-    name = models.CharField(max_length=120, blank=False, db_index=True, verbose_name="Field name")
-    verbose_name = models.CharField(max_length=120, blank=False, verbose_name="Verbose name")
-    typo = models.CharField(max_length=20, blank=False, db_index=True, verbose_name="Field type",
-        choices=single_list_to_tuple(FIELD_TYPES))
-    max_length = models.IntegerField(null=True, blank=True, verbose_name="Length")
-    blank = models.BooleanField(default=True, verbose_name="Blank")
-    choices = models.TextField(null=True, blank=True, verbose_name="Choices")
-    default_value = models.CharField(max_length=80, null=True, blank=True, verbose_name="Default value")
-
-    class Meta:
-        db_table = u'dynamic_field'
-
-
-# XXX: Charge memory with all dfields for prevent flood on db.
-# NOTE: this solution need to restart project on each new dfield add. Nasty!!
-dfields =  DynamicField.objects.all()
-def find_dfields(refer, name=None):
-    if name:
-        return [dfield for dfield in dfields \
-            if dfield.refer == refer and dfield.name == name]
-    return [dfield for dfield in dfields if dfield.refer == refer]
-
-# NOTE: Error happen on syncdb, because DynamicField's table does not exist.
-cursor = connection.cursor()
-cursor.execute("select count(*) from pg_tables where tablename='dynamic_field'")
-DYNAMIC_FIELD_TABLE_EXIST = (cursor.fetchone()[0] > 0)
-
-
 
 class HStoreModelMeta(models.Model.__metaclass__):
     def __new__(cls, name, bases, attrs):
@@ -149,7 +164,7 @@ class HStoreModelMeta(models.Model.__metaclass__):
             #print "called __setattr__(%r, %r)" % (key, value)
 
             if hasattr(self, '_dfields') and not key in dir(new_class):
-                # XXX: search for key on table, django will call this method on many times on 
+                # XXX: search for key on table, django will call this method on many times on
                 #      __init__
                 if find_dfields(refer=new_class.__name__, name=key):
                     if isinstance(value, (list, tuple)):
@@ -185,9 +200,9 @@ class HStoreModelMeta(models.Model.__metaclass__):
 
                 for metafield in metafields:
                     try:
-                        field_klass_name = FIELD_TYPES_DICT.get(metafield.typo, 
+                        field_klass_name = FIELD_TYPES_DICT.get(metafield.typo,
                                                                 FIELD_TYPE_DEFAULT)
-                        
+
                         #FIXME: eval is the evil, use module package
                         field_klass = eval(field_klass_name)
                         if metafield.choices == '':
